//+FHDR--------------------------------------------------------------------------------------------------------
//  Company: 
//-----------------------------------------------------------------------------------------------------------------
//  File Name             :    IIR_Filter_p1.v
//  Department            :    
//  Author                :    hdzhang
//  Author's Tel          :     
//-----------------------------------------------------------------------------------------------------------------
//  Relese History
//  Version     Date            Author          Description
//  0.0         2025-03-09      hdzhang
//2024-05-28 10:22:49 
//-----------------------------------------------------------------------------------------------------------------
//  Keywords            :       
//
//-----------------------------------------------------------------------------------------------------------------
//  Parameter
//
//-----------------------------------------------------------------------------------------------------------------
//  Purpose                 :
//                      
//-----------------------------------------------------------------------------------------------------------------
//  Target Device:        
//  Tool versions:        
//-----------------------------------------------------------------------------------------------------------------
//  Reuse Issues
//  Reset Strategy: 
//  Clock Domains: 
//  Critical Timing:
//  Asynchronous I/F:
//  Synthesizable (y/n): 
//  Other:
//-FHDR--------------------------------------------------------------------------------------------------------
module  IIR_Filter_p1 #(
 parameter  coef_width          = 32
,parameter  a_width		= 18
,parameter  b_width		= 18
,parameter  data_in_width       = 16 
,parameter  cascade_in_width	= 16 
,parameter  data_out_width	= 16
,parameter  temp_var_width	= data_out_width + 1
)
//H(z) = a / (1 - b*z^-1)
(
 input   rstn
,input   clk
,input   en
,input   signed [data_in_width-1 :0]   din_re     // Re(x(t))
,input   signed [cascade_in_width-1:0]   dout_r1_re // Re(y(t-1))
,input   signed [coef_width-1    :0]   a_re
,input   signed [coef_width-1    :0]   b_re

,output  signed [data_out_width-1:0]   dout_re // Re(y(t-16))
);


wire    signed  [temp_var_width-1  :0] x1_re;

wire    signed  [temp_var_width-1  :0] y1_re;
wire    signed  [temp_var_width    :0] y_re;

wire    signed  [data_out_width-1:0]   y_re_trunc;


// x1 = a * din delay M = a*x(t-8)
mult_real
#(
	 .A_width	(data_in_width	)  
	,.C_width	(a_width	) 
	,.o_width	(temp_var_width	) 
)
inst_c1 (
	.clk        	(clk            ),
	.rstn       	(rstn           ),
	.en         	(en             ),
	.din          	(din_re         ),
	.coef          	(a_re[coef_width-1 : coef_width-a_width]),
	.dout         	(x1_re          )
);


// y1 = b * dout_r1 delay M = b*y(t-9)
// y = y1+x1 = a*x(t-8)+b*y(t-9) = y(t-8) 
mult_real
#(
	 .A_width	(cascade_in_width	) 
	,.C_width	(b_width		) 
	,.o_width	(temp_var_width		) 
)
inst_c2 (
	.clk        	(clk        	),
	.rstn       	(rstn       	),
	.en         	(en             ),
 	.din          	(dout_r1_re     ),
	.coef          	(b_re[coef_width-1 : coef_width-b_width]),
	.dout         	(y1_re          )
);

assign  y_re       =   x1_re + y1_re;


// dout = round(y) delay M = round(y(t-16))
trunc #(
	 .diw	(temp_var_width+1		)
	,.msb	(temp_var_width-1		)
	,.lsb	(temp_var_width-data_out_width	)
) round_u1 (clk, rstn, en, y_re, y_re_trunc);

assign  dout_re = y_re_trunc;

endmodule
